@@ -1,9 +1,14 @@
 """Pymodbus ModbusSimulatorContext."""
+from __future__ import annotations
+
 import dataclasses
 import random
 import struct
+from collections.abc import Callable
 from datetime import datetime
-from typing import Any, Callable, Dict, List
+from typing import Any
+
+from pymodbus.datastore.context import ModbusBaseSlaveContext
 
 
 WORD_SIZE = 16
@@ -11,7 +16,7 @@ WORD_SIZE = 16
 
 @dataclasses.dataclass(frozen=True)
 class CellType:
-    """Define single cell types"""
+    """Define single cell types."""
 
     INVALID: int = 0
     BITS: int = 1
@@ -30,11 +35,23 @@ class Cell:
     access: bool = False
     value: int = 0
     action: int = 0
-    action_kwargs: Dict[str, Any] = None
+    action_parameters: dict[str, Any] | None = None
     count_read: int = 0
     count_write: int = 0
 
 
+class TextCell:  # pylint: disable=too-few-public-methods
+    """A textual representation of a single cell."""
+
+    type: str
+    access: str
+    value: str
+    action: str
+    action_parameters: str
+    count_read: str
+    count_write: str
+
+
 @dataclasses.dataclass
 class Label:  # pylint: disable=too-many-instance-attributes
     """Defines all dict values.
@@ -52,7 +69,7 @@ class Label:  # pylint: disable=too-many-instance-attributes
     increment: str = "increment"
     invalid: str = "invalid"
     ir_size: str = "ir size"
-    kwargs: str = "kwargs"
+    parameters: str = "parameters"
     method: str = "method"
     next: str = "next"
     none: str = "none"
@@ -92,8 +109,8 @@ class Setup:
     def __init__(self, runtime):
         """Initialize."""
         self.runtime = runtime
-        self.config = None
-        self.config_types = {
+        self.config = {}
+        self.config_types: dict[str, dict[str, Any]] = {
             Label.type_bits: {
                 Label.type: CellType.BITS,
                 Label.next: None,
@@ -131,7 +148,7 @@ class Setup:
             },
         }
 
-    def handle_type_bits(self, start, stop, value, action, action_kwargs):
+    def handle_type_bits(self, start, stop, value, action, action_parameters):
         """Handle type bits."""
         for reg in self.runtime.registers[start:stop]:
             if reg.type != CellType.INVALID:
@@ -139,9 +156,9 @@ class Setup:
             reg.value = value
             reg.type = CellType.BITS
             reg.action = action
-            reg.action_kwargs = action_kwargs
+            reg.action_parameters = action_parameters
 
-    def handle_type_uint16(self, start, stop, value, action, action_kwargs):
+    def handle_type_uint16(self, start, stop, value, action, action_parameters):
         """Handle type uint16."""
         for reg in self.runtime.registers[start:stop]:
             if reg.type != CellType.INVALID:
@@ -149,9 +166,9 @@ class Setup:
             reg.value = value
             reg.type = CellType.UINT16
             reg.action = action
-            reg.action_kwargs = action_kwargs
+            reg.action_parameters = action_parameters
 
-    def handle_type_uint32(self, start, stop, value, action, action_kwargs):
+    def handle_type_uint32(self, start, stop, value, action, action_parameters):
         """Handle type uint32."""
         regs_value = ModbusSimulatorContext.build_registers_from_value(value, True)
         for i in range(start, stop, 2):
@@ -161,11 +178,11 @@ class Setup:
             regs[0].value = regs_value[0]
             regs[0].type = CellType.UINT32
             regs[0].action = action
-            regs[0].action_kwargs = action_kwargs
+            regs[0].action_parameters = action_parameters
             regs[1].value = regs_value[1]
             regs[1].type = CellType.NEXT
 
-    def handle_type_float32(self, start, stop, value, action, action_kwargs):
+    def handle_type_float32(self, start, stop, value, action, action_parameters):
         """Handle type uint32."""
         regs_value = ModbusSimulatorContext.build_registers_from_value(value, False)
         for i in range(start, stop, 2):
@@ -175,11 +192,11 @@ class Setup:
             regs[0].value = regs_value[0]
             regs[0].type = CellType.FLOAT32
             regs[0].action = action
-            regs[0].action_kwargs = action_kwargs
+            regs[0].action_parameters = action_parameters
             regs[1].value = regs_value[1]
             regs[1].type = CellType.NEXT
 
-    def handle_type_string(self, start, stop, value, action, action_kwargs):
+    def handle_type_string(self, start, stop, value, action, action_parameters):
         """Handle type string."""
         regs = stop - start
         reg_len = regs * 2
@@ -197,10 +214,10 @@ class Setup:
             reg.type = CellType.NEXT
         self.runtime.registers[start].type = CellType.STRING
         self.runtime.registers[start].action = action
-        self.runtime.registers[start].action_kwargs = action_kwargs
+        self.runtime.registers[start].action_parameters = action_parameters
 
     def handle_setup_section(self):
-        """Load setup section"""
+        """Load setup section."""
         layout = Label.try_get(Label.setup, self.config)
         self.runtime.fc_offset = {key: 0 for key in range(25)}
         size_co = Label.try_get(Label.co_size, layout)
@@ -238,7 +255,7 @@ class Setup:
         del self.config[Label.setup]
 
     def handle_invalid_address(self):
-        """Handle invalid address"""
+        """Handle invalid address."""
         for entry in Label.try_get(Label.invalid, self.config):
             if isinstance(entry, int):
                 entry = [entry, entry]
@@ -252,7 +269,7 @@ class Setup:
         del self.config[Label.invalid]
 
     def handle_write_allowed(self):
-        """Handle write allowed"""
+        """Handle write allowed."""
         for entry in Label.try_get(Label.write, self.config):
             if isinstance(entry, int):
                 entry = [entry, entry]
@@ -269,7 +286,7 @@ class Setup:
         del self.config[Label.write]
 
     def handle_types(self):
-        """Handle the different types"""
+        """Handle the different types."""
         for section, type_entry in self.config_types.items():
             layout = Label.try_get(section, self.config)
             for entry in layout:
@@ -288,7 +305,7 @@ class Setup:
                     self.runtime.action_name_to_id[
                         entry.get(Label.action, type_entry[Label.action])
                     ],
-                    entry.get(Label.kwargs, None),
+                    entry.get(Label.parameters, None),
                 )
             del self.config[section]
 
@@ -356,8 +373,8 @@ class Setup:
             raise RuntimeError(f"INVALID key in setup: {self.config}")
 
 
-class ModbusSimulatorContext:
-    """Modbus simulator
+class ModbusSimulatorContext(ModbusBaseSlaveContext):
+    """Modbus simulator.
 
     :param config: A dict with structure as shown below.
     :param actions: A dict with "<name>": <function> structure.
@@ -418,12 +435,12 @@ class ModbusSimulatorContext:
             "write": [   --> allow write, efault is ReadOnly
                 [5, 5]  --> start, end bytes, repeated as needed
             ],
-            "bits": [  --> Define bits (1 register == 1 byte)
+            "bits": [  --> Define bits (1 register == 2 bytes)
                 [30, 31],  --> start, end registers, repeated as needed
                 {"addr": [32, 34], "value": 0xF1},  --> with value
                 {"addr": [35, 36], "action": "increment"},  --> with action
                 {"addr": [37, 38], "action": "increment", "value": 0xF1}  --> with action and value
-                {"addr": [37, 38], "action": "increment", "kwargs": {"min": 0, "max": 100}}  --> with action with arguments
+                {"addr": [37, 38], "action": "increment", "parameters": {"min": 0, "max": 100}}  --> with action with arguments
             ],
             "uint16": [  --> Define uint16 (1 register == 2 bytes)
                 --> same as type_bits
@@ -452,18 +469,18 @@ class ModbusSimulatorContext:
     start_time = int(datetime.now().timestamp())
 
     def __init__(
-        self, config: Dict[str, Any], custom_actions: Dict[str, Callable]
+        self, config: dict[str, Any], custom_actions: dict[str, Callable] | None
     ) -> None:
         """Initialize."""
-        self.registers: List[int] = []
-        self.fc_offset: Dict[int, int] = {}
+        self.registers: list[Cell] = []
+        self.fc_offset: dict[int, int] = {}
         self.register_count = 0
         self.type_exception = False
-        self.action_name_to_id: Dict[str, int] = {}
-        self.action_id_to_name: List[str] = []
-        self.action_methods: List[Callable] = []
-        self.registerType_name_to_id: Dict[str, int] = {}
-        self.registerType_id_to_name: List[str] = []
+        self.action_name_to_id: dict[str, int] = {}
+        self.action_id_to_name: list[str] = []
+        self.action_methods: list[Callable] = []
+        self.registerType_name_to_id: dict[str, int] = {}
+        self.registerType_id_to_name: list[str] = []
         Setup(self).setup(config, custom_actions)
 
     # --------------------------------------------
@@ -472,14 +489,14 @@ class ModbusSimulatorContext:
     def get_text_register(self, register):
         """Get raw register."""
         reg = self.registers[register]
-        text_cell = Cell()
+        text_cell = TextCell()
         text_cell.type = self.registerType_id_to_name[reg.type]
         text_cell.access = str(reg.access)
         text_cell.count_read = str(reg.count_read)
         text_cell.count_write = str(reg.count_write)
         text_cell.action = self.action_id_to_name[reg.action]
-        if reg.action_kwargs:
-            text_cell.action = f"{text_cell.action}({reg.action_kwargs})"
+        if reg.action_parameters:
+            text_cell.action = f"{text_cell.action}({reg.action_parameters})"
         if reg.type in (CellType.INVALID, CellType.UINT16, CellType.NEXT):
             text_cell.value = str(reg.value)
             build_len = 0
@@ -572,9 +589,9 @@ class ModbusSimulatorContext:
             real_address = self.fc_offset[func_code] + address
             for i in range(real_address, real_address + count):
                 reg = self.registers[i]
-                kwargs = reg.action_kwargs if reg.action_kwargs else {}
+                parameters = reg.action_parameters if reg.action_parameters else {}
                 if reg.action:
-                    self.action_methods[reg.action](self.registers, i, reg, **kwargs)
+                    self.action_methods[reg.action](self.registers, i, reg, **parameters)
                 self.registers[i].count_read += 1
                 result.append(reg.value)
         else:
@@ -585,8 +602,9 @@ class ModbusSimulatorContext:
             for i in range(real_address, real_address + reg_count):
                 reg = self.registers[i]
                 if reg.action:
+                    parameters = reg.action_parameters or {}
                     self.action_methods[reg.action](
-                        self.registers, i, reg, reg.action_kwargs
+                        self.registers, i, reg, **parameters
                     )
                 self.registers[i].count_read += 1
                 while count and bit_index < 16:
@@ -689,7 +707,7 @@ class ModbusSimulatorContext:
             reg2.value = new_regs[1]
 
     @classmethod
-    def action_timestamp(cls, registers, inx, _cell, **_kwargs):
+    def action_timestamp(cls, registers, inx, _cell, **_parameters):
         """Set current time.
 
         :meta private:
@@ -704,7 +722,7 @@ class ModbusSimulatorContext:
         registers[inx + 6].value = system_time.second
 
     @classmethod
-    def action_reset(cls, _registers, _inx, _cell, **_kwargs):
+    def action_reset(cls, _registers, _inx, _cell, **_parameters):
         """Reboot server.
 
         :meta private:
@@ -712,7 +730,7 @@ class ModbusSimulatorContext:
         raise RuntimeError("RESET server")
 
     @classmethod
-    def action_uptime(cls, registers, inx, cell, **_kwargs):
+    def action_uptime(cls, registers, inx, cell, **_parameters):
         """Return uptime in seconds.
 
         :meta private:
@@ -734,12 +752,12 @@ class ModbusSimulatorContext:
     # Internal helper methods
     # --------------------------------------------
 
-    def validate_type(self, func_code, real_address, count):
-        """Check if request is done against correct type
+    def validate_type(self, func_code, real_address, count) -> bool:
+        """Check if request is done against correct type.
 
         :meta private:
         """
-
+        check: tuple
         if func_code in self._bits_func_code:
             # Bit access
             check = (CellType.BITS, -1)
@@ -762,7 +780,7 @@ class ModbusSimulatorContext:
 
     @classmethod
     def build_registers_from_value(cls, value, is_int):
-        """Build registers from int32 or float32"""
+        """Build registers from int32 or float32."""
         regs = [0, 0]
         if is_int:
             value_bytes = int.to_bytes(value, 4, "big")
@@ -774,7 +792,7 @@ class ModbusSimulatorContext:
 
     @classmethod
     def build_value_from_registers(cls, registers, is_int):
-        """Build registers from int32 or float32"""
+        """Build int32 or float32 value from registers."""
         value_bytes = int.to_bytes(registers[0], 2, "big") + int.to_bytes(
             registers[1], 2, "big"
         )
