@@ -1,29 +1,21 @@
 """Implementation of a Threaded Modbus Server."""
 # pylint: disable=missing-type-doc
+from __future__ import annotations
+
 import asyncio
-import time
+import os
 import traceback
 from contextlib import suppress
-from typing import Union
 
 from pymodbus.datastore import ModbusServerContext
 from pymodbus.device import ModbusControlBlock, ModbusDeviceIdentification
-from pymodbus.exceptions import NoSuchSlaveException
-from pymodbus.factory import ServerDecoder
-from pymodbus.framer import ModbusFramer
+from pymodbus.exceptions import ModbusException, NoSuchSlaveException
+from pymodbus.framer import FRAMER_NAME_TO_CLASS, FramerBase, FramerType
 from pymodbus.logging import Log
+from pymodbus.pdu import DecodePDU
 from pymodbus.pdu import ModbusExceptions as merror
-from pymodbus.transaction import (
-    ModbusAsciiFramer,
-    ModbusRtuFramer,
-    ModbusSocketFramer,
-    ModbusTlsFramer,
-)
-from pymodbus.transport.transport import CommParams, CommType, ModbusProtocol
-
-
-with suppress(ImportError):
-    pass
+from pymodbus.pdu.pdu import ExceptionResponse
+from pymodbus.transport import CommParams, CommType, ModbusProtocol
 
 
 # --------------------------------------------------------------------------- #
@@ -55,9 +47,11 @@ class ModbusServerRequestHandler(ModbusProtocol):
         super().__init__(params, True)
         self.server = owner
         self.running = False
-        self.receive_queue = asyncio.Queue()
+        self.receive_queue: asyncio.Queue = asyncio.Queue()
         self.handler_task = None  # coroutine to be run on asyncio loop
-        self.framer: ModbusFramer = None
+        self.databuffer = b''
+        self.framer: FramerBase
+        self.loop = asyncio.get_running_loop()
 
     def _log_exception(self):
         """Show log exception."""
@@ -65,25 +59,32 @@ class ModbusServerRequestHandler(ModbusProtocol):
             "Handler for stream [{}] has been canceled", self.comm_params.comm_name
         )
 
+    def callback_new_connection(self) -> ModbusProtocol:
+        """Call when listener receive new connection request."""
+        Log.debug("callback_new_connection called")
+        return ModbusServerRequestHandler(self)
+
     def callback_connected(self) -> None:
         """Call when connection is succcesfull."""
+        slaves = self.server.context.slaves()
+        if self.server.broadcast_enable:
+            if 0 not in slaves:
+                slaves.append(0)
         try:
             self.running = True
-            self.framer = self.server.framer(
-                self.server.decoder,
-                client=None,
-            )
+            self.framer = self.server.framer(self.server.decoder)
 
             # schedule the connection handler on the event loop
             self.handler_task = asyncio.create_task(self.handle())
-        except Exception as exc:  # pragma: no cover pylint: disable=broad-except
+            self.handler_task.set_name("server connection handler")
+        except Exception as exc:  # pylint: disable=broad-except
             Log.error(
                 "Server callback_connected exception: {}; {}",
                 exc,
                 traceback.format_exc(),
             )
 
-    def callback_disconnected(self, call_exc: Exception) -> None:
+    def callback_disconnected(self, call_exc: Exception | None) -> None:
         """Call when connection is lost."""
         try:
             if self.handler_task:
@@ -108,48 +109,47 @@ class ModbusServerRequestHandler(ModbusProtocol):
 
     async def inner_handle(self):
         """Handle handler."""
-        slaves = self.server.context.slaves()
         # this is an asyncio.Queue await, it will never fail
         data = await self._recv_()
         if isinstance(data, tuple):
             # addr is populated when talking over UDP
             data, *addr = data
         else:
-            addr = (None,)  # empty tuple
+            addr = [None]
 
         # if broadcast is enabled make sure to
         # process requests to address 0
-        if self.server.broadcast_enable:  # pragma: no cover
-            if 0 not in slaves:
-                slaves.append(0)
+        self.databuffer += data
+        Log.debug("Handling data: {}", self.databuffer, ":hex")
+        try:
+            used_len, pdu = self.framer.processIncomingFrame(self.databuffer)
+        except ModbusException:
+            pdu = ExceptionResponse(
+                40,
+                exception_code=merror.IllegalFunction
+            )
+            self.server_send(pdu, 0)
+            pdu = None
+            used_len = len(self.databuffer)
+        self.databuffer = self.databuffer[used_len:]
+        if pdu:
+           self.execute(pdu, *addr)
 
-        Log.debug("Handling data: {}", data, ":hex")
-
-        single = self.server.context.single
-        self.framer.processIncomingPacket(
-            data=data,
-            callback=lambda x: self.execute(x, *addr),
-            slave=slaves,
-            single=single,
-        )
-
-    async def handle(self):
-        """Return Asyncio coroutine which represents a single conversation.
-
-        between the modbus slave and master
+    async def handle(self) -> None:
+        """Coroutine which represents a single master <=> slave conversation.
 
         Once the client connection is established, the data chunks will be
         fed to this coroutine via the asyncio.Queue object which is fed by
         the ModbusServerRequestHandler class's callback Future.
 
-        This callback future gets data from either
+        This callback future gets data from either asyncio.BaseProtocol.data_received
+        or asyncio.DatagramProtocol.datagram_received.
 
         This function will execute without blocking in the while-loop and
         yield to the asyncio event loop when the frame is exhausted.
         As a result, multiple clients can be interleaved without any
         interference between them.
         """
-        reset_frame = False
         while self.running:
             try:
                 await self.inner_handle()
@@ -159,23 +159,15 @@ class ModbusServerRequestHandler(ModbusProtocol):
                     self._log_exception()
                     self.running = False
             except Exception as exc:  # pylint: disable=broad-except
-                # force TCP socket termination as processIncomingPacket
+                # force TCP socket termination as framer
                 # should handle application layer errors
-                # for UDP sockets, simply reset the frame
-                if isinstance(self, ModbusServerRequestHandler):
-                    Log.error(
-                        'Unknown exception "{}" on stream {} forcing disconnect',
-                        exc,
-                        self.comm_params.comm_name,
-                    )
-                    self.transport_close()
-                else:
-                    Log.error("Unknown error occurred {}", exc)
-                    reset_frame = True  # graceful recovery
-            finally:
-                if reset_frame:
-                    self.framer.resetFrame()
-                    reset_frame = False
+                Log.error(
+                    'Unknown exception "{}" on stream {} forcing disconnect',
+                    exc,
+                    self.comm_params.comm_name,
+                )
+                self.close()
+                self.callback_disconnected(exc)
 
     def execute(self, request, *addr):
         """Call with the resulting message.
@@ -186,6 +178,9 @@ class ModbusServerRequestHandler(ModbusProtocol):
         if self.server.request_tracer:
             self.server.request_tracer(request, *addr)
 
+        asyncio.run_coroutine_threadsafe(self._async_execute(request, *addr), self.loop)
+
+    async def _async_execute(self, request, *addr):
         broadcast = False
         try:
             if self.server.broadcast_enable and not request.slave_id:
@@ -193,10 +188,11 @@ class ModbusServerRequestHandler(ModbusProtocol):
                 # if broadcasting then execute on all slave contexts,
                 # note response will be ignored
                 for slave_id in self.server.context.slaves():
-                    response = request.execute(self.server.context[slave_id])
+                    response = await request.update_datastore(self.server.context[slave_id])
             else:
                 context = self.server.context[request.slave_id]
-                response = request.execute(context)
+                response = await request.update_datastore(context)
+
         except NoSuchSlaveException:
             Log.error("requested slave does not exist: {}", request.slave_id)
             if self.server.ignore_missing_slaves:
@@ -216,19 +212,19 @@ class ModbusServerRequestHandler(ModbusProtocol):
             skip_encoding = False
             if self.server.response_manipulator:
                 response, skip_encoding = self.server.response_manipulator(response)
-            self.send(response, *addr, skip_encoding=skip_encoding)
+            self.server_send(response, *addr, skip_encoding=skip_encoding)
 
-    def send(self, message, addr, **kwargs):
+    def server_send(self, message, addr, **kwargs):
         """Send message."""
         if kwargs.get("skip_encoding", False):
-            self.transport_send(message, addr=addr)
-        elif message.should_respond:
-            pdu = self.framer.buildPacket(message)
-            self.transport_send(pdu, addr=addr)
-        else:
+            self.send(message, addr=addr)
+        if not message:
             Log.debug("Skipping sending response!!")
+        else:
+            pdu = self.framer.buildFrame(message)
+            self.send(pdu, addr=addr)
 
-    async def _recv_(self):  # pragma: no cover
+    async def _recv_(self):
         """Receive data from the network."""
         try:
             result = await self.receive_queue.get()
@@ -237,9 +233,9 @@ class ModbusServerRequestHandler(ModbusProtocol):
             result = None
         return result
 
-    def callback_data(self, data: bytes, addr: tuple = None) -> int:
+    def callback_data(self, data: bytes, addr: tuple | None = ()) -> int:
         """Handle received data."""
-        if addr:
+        if addr != ():
             self.receive_queue.put_nowait((data, addr))
         else:
             self.receive_queue.put_nowait(data)
@@ -251,7 +247,74 @@ class ModbusServerRequestHandler(ModbusProtocol):
 # --------------------------------------------------------------------------- #
 
 
-class ModbusTcpServer(ModbusProtocol):
+class ModbusBaseServer(ModbusProtocol):
+    """Common functionality for all server classes."""
+
+    def __init__(
+        self,
+        params: CommParams,
+        context,
+        ignore_missing_slaves,
+        broadcast_enable,
+        response_manipulator,
+        request_tracer,
+        identity,
+        framer,
+    ) -> None:
+        """Initialize base server."""
+        super().__init__(
+            params,
+            True,
+        )
+        self.loop = asyncio.get_running_loop()
+        self.decoder = DecodePDU(True)
+        self.context = context or ModbusServerContext()
+        self.control = ModbusControlBlock()
+        self.ignore_missing_slaves = ignore_missing_slaves
+        self.broadcast_enable = broadcast_enable
+        self.response_manipulator = response_manipulator
+        self.request_tracer = request_tracer
+        self.handle_local_echo = False
+        if isinstance(identity, ModbusDeviceIdentification):
+            self.control.Identity.update(identity)
+
+        self.framer = FRAMER_NAME_TO_CLASS[framer]
+        self.serving: asyncio.Future = asyncio.Future()
+
+    def callback_new_connection(self):
+        """Handle incoming connect."""
+        return ModbusServerRequestHandler(self)
+
+    async def shutdown(self):
+        """Close server."""
+        if not self.serving.done():
+            self.serving.set_result(True)
+        self.close()
+
+    async def serve_forever(self):
+        """Start endless loop."""
+        if self.transport:
+            raise RuntimeError(
+                "Can't call serve_forever on an already running server object"
+            )
+        await self.listen()
+        Log.info("Server listening.")
+        await self.serving
+        Log.info("Server graceful shutdown.")
+
+    def callback_connected(self) -> None:
+        """Call when connection is succcesfull."""
+
+    def callback_disconnected(self, exc: Exception | None) -> None:
+        """Call when connection is lost."""
+        Log.debug("callback_disconnected called: {}", exc)
+
+    def callback_data(self, data: bytes, addr: tuple | None = None) -> int:
+        """Handle received data."""
+        Log.debug("callback_data called: {} addr={}", data, ":hex", addr)
+        return 0
+
+class ModbusTcpServer(ModbusBaseServer):
     """A modbus threaded tcp socket server.
 
     We inherit and overload the socket server so that we
@@ -262,7 +325,7 @@ class ModbusTcpServer(ModbusProtocol):
     def __init__(
         self,
         context,
-        framer=None,
+        framer=FramerType.SOCKET,
         identity=None,
         address=("", 502),
         ignore_missing_slaves=False,
@@ -299,60 +362,16 @@ class ModbusTcpServer(ModbusProtocol):
             ),
         )
         params.source_address = address
-
         super().__init__(
             params,
-            True,
+            context,
+            ignore_missing_slaves,
+            broadcast_enable,
+            response_manipulator,
+            request_tracer,
+            identity,
+            framer,
         )
-        self.decoder = ServerDecoder()
-        self.framer = framer or ModbusSocketFramer
-        self.context = context or ModbusServerContext()
-        self.control = ModbusControlBlock()
-        self.ignore_missing_slaves = ignore_missing_slaves
-        self.broadcast_enable = broadcast_enable
-        self.response_manipulator = response_manipulator
-        self.request_tracer = request_tracer
-        if isinstance(identity, ModbusDeviceIdentification):
-            self.control.Identity.update(identity)
-
-        # asyncio future that will be done once server has started
-        self.serving = asyncio.Future()
-        self.serving_done = asyncio.Future()
-        # constructors cannot be declared async, so we have to
-        # defer the initialization of the server
-        self.handle_local_echo = False
-
-    def handle_new_connection(self):
-        """Handle incoming connect."""
-        return ModbusServerRequestHandler(self)
-
-    async def serve_forever(self):
-        """Start endless loop."""
-        if self.transport:
-            raise RuntimeError(
-                "Can't call serve_forever on an already running server object"
-            )
-
-        await self.transport_listen()
-        self.serving.set_result(True)
-        Log.info("Server(TCP) listening.")
-        try:
-            await self.transport.serve_forever()
-        except asyncio.exceptions.CancelledError:
-            self.serving_done.set_result(False)
-            raise
-        except Exception as exc:  # pylint: disable=broad-except
-            Log.error("Server unexpected exception {}", exc)
-        self.serving_done.set_result(True)
-        Log.info("Server graceful shutdown.")
-
-    async def shutdown(self):
-        """Shutdown server."""
-        await self.server_close()
-
-    async def server_close(self):
-        """Close server."""
-        self.transport_close()
 
 
 class ModbusTlsServer(ModbusTcpServer):
@@ -366,7 +385,7 @@ class ModbusTlsServer(ModbusTcpServer):
     def __init__(  # pylint: disable=too-many-arguments
         self,
         context,
-        framer=None,
+        framer=FramerType.TLS,
         identity=None,
         address=("", 502),
         sslctx=None,
@@ -421,7 +440,7 @@ class ModbusTlsServer(ModbusTcpServer):
         )
 
 
-class ModbusUdpServer(ModbusProtocol):
+class ModbusUdpServer(ModbusBaseServer):
     """A modbus threaded udp socket server.
 
     We inherit and overload the socket server so that we
@@ -432,7 +451,7 @@ class ModbusUdpServer(ModbusProtocol):
     def __init__(
         self,
         context,
-        framer=None,
+        framer=FramerType.SOCKET,
         identity=None,
         address=("", 502),
         ignore_missing_slaves=False,
@@ -449,8 +468,6 @@ class ModbusUdpServer(ModbusProtocol):
         :param framer: The framer strategy to use
         :param identity: An optional identify structure
         :param address: An optional (interface, port) to bind to.
-        :param handler: A handler for each client session; default is
-                            ModbusDisonnectedRequestHandler
         :param ignore_missing_slaves: True to not send errors on a request
                             to a missing slave
         :param broadcast_enable: True to treat slave_id 0 as broadcast address,
@@ -469,63 +486,17 @@ class ModbusUdpServer(ModbusProtocol):
                 reconnect_delay_max=0.0,
                 timeout_connect=0.0,
             ),
-            True,
+            context,
+            ignore_missing_slaves,
+            broadcast_enable,
+            response_manipulator,
+            request_tracer,
+            identity,
+            framer,
         )
 
-        self.loop = asyncio.get_running_loop()
-        self.decoder = ServerDecoder()
-        self.framer = framer or ModbusSocketFramer
-        self.context = context or ModbusServerContext()
-        self.control = ModbusControlBlock()
-        self.ignore_missing_slaves = ignore_missing_slaves
-        self.broadcast_enable = broadcast_enable
-        self.response_manipulator = response_manipulator
-        self.request_tracer = request_tracer
-        if isinstance(identity, ModbusDeviceIdentification):
-            self.control.Identity.update(identity)
-        self.stop_serving = self.loop.create_future()
-        # asyncio future that will be done once server has started
-        self.serving = asyncio.Future()
-        self.serving_done = asyncio.Future()
-        self.request_tracer = None
-        self.handle_local_echo = False
 
-    def handle_new_connection(self):
-        """Handle incoming connect."""
-        return ModbusServerRequestHandler(self)
-
-    async def serve_forever(self):
-        """Start endless loop."""
-        if self.transport:
-            raise RuntimeError(
-                "Can't call serve_forever on an already running server object"
-            )
-        try:
-            await self.transport_listen()
-        except asyncio.exceptions.CancelledError:
-            self.serving_done.set_result(False)
-            raise
-        except Exception as exc:
-            Log.error("Server unexpected exception {}", exc)
-            self.serving_done.set_result(False)
-            raise RuntimeError(exc) from exc
-        Log.info("Server(UDP) listening.")
-        self.serving.set_result(True)
-        await self.stop_serving
-        self.serving_done.set_result(True)
-
-    async def shutdown(self):
-        """Shutdown server."""
-        await self.server_close()
-
-    async def server_close(self):
-        """Close server."""
-        self.transport_close()
-        if not self.stop_serving.done():
-            self.stop_serving.set_result(True)
-
-
-class ModbusSerialServer(ModbusProtocol):
+class ModbusSerialServer(ModbusBaseServer):
     """A modbus threaded serial socket server.
 
     We inherit and overload the socket server so that we
@@ -534,14 +505,14 @@ class ModbusSerialServer(ModbusProtocol):
     """
 
     def __init__(
-        self, context, framer=ModbusRtuFramer, identity=None, **kwargs
-    ):  # pragma: no cover
+        self, context, framer=FramerType.RTU, identity=None, **kwargs
+    ):
         """Initialize the socket server.
 
         If the identity structure is not passed in, the ModbusControlBlock
         uses its own empty structure.
         :param context: The ModbusServerContext datastore
-        :param framer: The framer strategy to use, default ModbusRtuFramer
+        :param framer: The framer strategy to use, default FramerType.RTU
         :param identity: An optional identify structure
         :param port: The serial port to attach to
         :param stopbits: The number of stop bits to use
@@ -557,9 +528,10 @@ class ModbusSerialServer(ModbusProtocol):
         :param reconnect_delay: reconnect delay in seconds
         :param response_manipulator: Callback method for
                     manipulating the response
+        :param request_tracer: Callback method for tracing
         """
         super().__init__(
-            CommParams(
+            params=CommParams(
                 comm_type=CommType.SERIAL,
                 comm_name="server_listener",
                 reconnect_delay=kwargs.get("reconnect_delay", 2),
@@ -571,56 +543,15 @@ class ModbusSerialServer(ModbusProtocol):
                 baudrate=kwargs.get("baudrate", 19200),
                 stopbits=kwargs.get("stopbits", 1),
             ),
-            True,
+            context=context,
+            ignore_missing_slaves=kwargs.get("ignore_missing_slaves", False),
+            broadcast_enable=kwargs.get("broadcast_enable", False),
+            response_manipulator=kwargs.get("response_manipulator", None),
+            request_tracer=kwargs.get("request_tracer", None),
+            identity=kwargs.get("identity", None),
+            framer=framer,
         )
-
-        self.loop = kwargs.get("loop") or asyncio.get_event_loop()
         self.handle_local_echo = kwargs.get("handle_local_echo", False)
-        self.ignore_missing_slaves = kwargs.get("ignore_missing_slaves", False)
-        self.broadcast_enable = kwargs.get("broadcast_enable", False)
-        self.framer = framer or ModbusRtuFramer
-        self.decoder = ServerDecoder()
-        self.context = context or ModbusServerContext()
-        self.response_manipulator = kwargs.get("response_manipulator", None)
-        self.control = ModbusControlBlock()
-        if isinstance(identity, ModbusDeviceIdentification):
-            self.control.Identity.update(identity)
-        self.request_tracer = None
-        self.server = None
-        self.control = ModbusControlBlock()
-        identity = kwargs.get("identity")
-        if isinstance(identity, ModbusDeviceIdentification):
-            self.control.Identity.update(identity)
-
-    async def start(self):
-        """Start connecting."""
-
-    def handle_new_connection(self):
-        """Handle incoming connect."""
-        return ModbusServerRequestHandler(self)
-
-    async def shutdown(self):
-        """Terminate server."""
-        self.transport_close()
-        if self.server:
-            self.server.close()
-            await asyncio.wait_for(self.server.wait_closed(), 10)
-            self.server = None
-
-    async def serve_forever(self):
-        """Start endless loop."""
-        if self.server:
-            raise RuntimeError(
-                "Can't call serve_forever on an already running server object"
-            )
-        Log.info("Server(Serial) listening.")
-        await self.transport_listen()
-        try:
-            await self.transport.serve_forever()
-        except asyncio.exceptions.CancelledError:
-            raise
-        except Exception as exc:  # pylint: disable=broad-except
-            Log.error("Server unexpected exception {}", exc)
 
 
 # --------------------------------------------------------------------------- #
@@ -634,7 +565,7 @@ class _serverList:
     :meta private:
     """
 
-    active_server: Union[ModbusTcpServer, ModbusUdpServer, ModbusSerialServer] = None
+    active_server: ModbusTcpServer | ModbusUdpServer | ModbusSerialServer
 
     def __init__(self, server):
         """Register new server."""
@@ -642,22 +573,21 @@ class _serverList:
         self.loop = asyncio.get_event_loop()
 
     @classmethod
-    async def run(cls, server, custom_functions):
+    async def run(cls, server, custom_functions) -> None:
         """Help starting/stopping server."""
         for func in custom_functions:
             server.decoder.register(func)
-        cls.active_server = _serverList(server)
+        cls.active_server = _serverList(server)  # type: ignore[assignment]
         with suppress(asyncio.exceptions.CancelledError):
             await server.serve_forever()
 
     @classmethod
-    async def async_stop(cls):
+    async def async_stop(cls) -> None:
         """Wait for server stop."""
         if not cls.active_server:
             raise RuntimeError("ServerAsyncStop called without server task active.")
-        await cls.active_server.server.shutdown()
-        await asyncio.sleep(1)
-        cls.active_server = None
+        await cls.active_server.server.shutdown()  # type: ignore[union-attr]
+        cls.active_server = None  # type: ignore[assignment]
 
     @classmethod
     def stop(cls):
@@ -668,8 +598,8 @@ class _serverList:
         if not cls.active_server.loop.is_running():
             Log.info("ServerStop called with loop stopped.")
             return
-        asyncio.run_coroutine_threadsafe(cls.async_stop(), cls.active_server.loop)
-        time.sleep(10)
+        future = asyncio.run_coroutine_threadsafe(cls.async_stop(), cls.active_server.loop)
+        future.result(timeout=10 if os.name == 'nt' else 0.1)
 
 
 async def StartAsyncTcpServer(  # pylint: disable=invalid-name,dangerous-default-value
@@ -690,7 +620,7 @@ async def StartAsyncTcpServer(  # pylint: disable=invalid-name,dangerous-default
     """
     kwargs.pop("host", None)
     server = ModbusTcpServer(
-        context, kwargs.pop("framer", ModbusSocketFramer), identity, address, **kwargs
+        context, kwargs.pop("framer", FramerType.SOCKET), identity, address, **kwargs
     )
     await _serverList.run(server, custom_functions)
 
@@ -722,7 +652,7 @@ async def StartAsyncTlsServer(  # pylint: disable=invalid-name,dangerous-default
     kwargs.pop("host", None)
     server = ModbusTlsServer(
         context,
-        kwargs.pop("framer", ModbusTlsFramer),
+        kwargs.pop("framer", FramerType.TLS),
         identity,
         address,
         sslctx,
@@ -752,7 +682,7 @@ async def StartAsyncUdpServer(  # pylint: disable=invalid-name,dangerous-default
     """
     kwargs.pop("host", None)
     server = ModbusUdpServer(
-        context, kwargs.pop("framer", ModbusSocketFramer), identity, address, **kwargs
+        context, kwargs.pop("framer", FramerType.SOCKET), identity, address, **kwargs
     )
     await _serverList.run(server, custom_functions)
 
@@ -762,7 +692,7 @@ async def StartAsyncSerialServer(  # pylint: disable=invalid-name,dangerous-defa
     identity=None,
     custom_functions=[],
     **kwargs,
-):  # pragma: no cover
+):
     """Start and run a serial modbus server.
 
     :param context: The ModbusServerContext datastore
@@ -772,9 +702,8 @@ async def StartAsyncSerialServer(  # pylint: disable=invalid-name,dangerous-defa
     :param kwargs: The rest
     """
     server = ModbusSerialServer(
-        context, kwargs.pop("framer", ModbusAsciiFramer), identity=identity, **kwargs
+        context, kwargs.pop("framer", FramerType.RTU), identity=identity, **kwargs
     )
-    await server.start()
     await _serverList.run(server, custom_functions)
 
 
