@@ -1,4 +1,4 @@
-"""Datastore simulator, custom actions"""
+"""Datastore simulator, custom actions."""
 
 
 def device_reset(_registers, _inx, _cell):
