@@ -1,39 +1,37 @@
 """HTTP server for modbus simulator."""
+from __future__ import annotations
+
 import asyncio
+import contextlib
 import dataclasses
 import importlib
 import json
 import os
-from time import time
-from typing import List
+from time import sleep
+from typing import TYPE_CHECKING
 
 
 try:
     from aiohttp import web
-except ImportError:
-    web = None
 
-import contextlib
+    AIOHTTP_MISSING = False
+except ImportError:
+    AIOHTTP_MISSING = True
+    if TYPE_CHECKING:  # always False at runtime
+        # type checkers do not understand the Raise RuntimeError in __init__()
+        from aiohttp import web
 
 from pymodbus.datastore import ModbusServerContext, ModbusSimulatorContext
 from pymodbus.datastore.simulator import Label
 from pymodbus.device import ModbusDeviceIdentification
-from pymodbus.factory import ServerDecoder
 from pymodbus.logging import Log
-from pymodbus.pdu import ExceptionResponse
-from pymodbus.server import (
+from pymodbus.pdu import DecodePDU, ExceptionResponse
+from pymodbus.server.async_io import (
     ModbusSerialServer,
     ModbusTcpServer,
     ModbusTlsServer,
     ModbusUdpServer,
 )
-from pymodbus.transaction import (
-    ModbusAsciiFramer,
-    ModbusBinaryFramer,
-    ModbusRtuFramer,
-    ModbusSocketFramer,
-    ModbusTlsFramer,
-)
 
 
 MAX_FILTER = 1000
@@ -47,7 +45,7 @@ RESPONSE_JUNK = 3
 
 @dataclasses.dataclass()
 class CallTracer:
-    """Define call/response traces"""
+    """Define call/response traces."""
 
     call: bool = False
     fc: int = -1
@@ -58,7 +56,7 @@ class CallTracer:
 
 @dataclasses.dataclass()
 class CallTypeMonitor:
-    """Define Request/Response monitor"""
+    """Define Request/Response monitor."""
 
     active: bool = False
     trace_response: bool = False
@@ -71,7 +69,7 @@ class CallTypeMonitor:
 
 @dataclasses.dataclass()
 class CallTypeResponse:
-    """Define Response manipulation"""
+    """Define Response manipulation."""
 
     active: int = RESPONSE_INACTIVE
     split: int = 0
@@ -103,17 +101,17 @@ class ModbusSimulatorServer:
 
     Example::
 
-        from pymodbus.server import StartAsyncSimulatorServer
+        from pymodbus.server import ModbusSimulatorServer
 
         async def run():
-            simulator = StartAsyncSimulatorServer(
+            simulator = ModbusSimulatorServer(
                 modbus_server="my server",
                 modbus_device="my device",
                 http_host="localhost",
                 http_port=8080)
-            await simulator.start()
+            await simulator.run_forever(only_start=True)
             ...
-            await simulator.close()
+            await simulator.stop()
     """
 
     def __init__(
@@ -124,11 +122,14 @@ class ModbusSimulatorServer:
         http_port: int = 8080,
         log_file: str = "server.log",
         json_file: str = "setup.json",
-        custom_actions_module: str = None,
+        custom_actions_module: str | None = None,
     ):
         """Initialize http interface."""
-        if not web:
-            raise RuntimeError("aiohttp not installed!")
+        if AIOHTTP_MISSING:
+            raise RuntimeError(
+                "Simulator server requires aiohttp. "
+                'Please install with "pip install aiohttp" and try again.'
+            )
         with open(json_file, encoding="utf-8") as file:
             setup = json.load(file)
 
@@ -138,18 +139,11 @@ class ModbusSimulatorServer:
             "tls": ModbusTlsServer,
             "udp": ModbusUdpServer,
         }
-        framer_class = {
-            "ascii": ModbusAsciiFramer,
-            "binary": ModbusBinaryFramer,
-            "rtu": ModbusRtuFramer,
-            "socket": ModbusSocketFramer,
-            "tls": ModbusTlsFramer,
-        }
         if custom_actions_module:
             actions_module = importlib.import_module(custom_actions_module)
             custom_actions_dict = actions_module.custom_actions_dict
         else:
-            custom_actions_dict = None
+            custom_actions_dict = {}
         server = setup["server_list"][modbus_server]
         if server["comm"] != "serial":
             server["address"] = (server["host"], server["port"])
@@ -157,19 +151,27 @@ class ModbusSimulatorServer:
             del server["port"]
         device = setup["device_list"][modbus_device]
         self.datastore_context = ModbusSimulatorContext(
-            device, custom_actions_dict or None
+            device, custom_actions_dict or {}
         )
-        datastore = ModbusServerContext(slaves=self.datastore_context, single=True)
+        datastore = None
+        if "device_id" in server:
+            # Designated ModBus unit address. Will only serve data if the address matches
+            datastore = ModbusServerContext(slaves={int(server["device_id"]): self.datastore_context}, single=False)
+        else:
+            # Will server any request regardless of addressing
+            datastore = ModbusServerContext(slaves=self.datastore_context, single=True)
+
         comm = comm_class[server.pop("comm")]
-        framer = framer_class[server.pop("framer")]
+        framer = server.pop("framer")
         if "identity" in server:
             server["identity"] = ModbusDeviceIdentification(
                 info_name=server["identity"]
             )
         self.modbus_server = comm(framer=framer, context=datastore, **server)
-
+        self.serving: asyncio.Future = asyncio.Future()
         self.log_file = log_file
-        self.site = None
+        self.site: web.TCPSite | None = None
+        self.runner: web.AppRunner
         self.http_host = http_host
         self.http_port = http_port
         self.web_path = os.path.join(os.path.dirname(__file__), "web")
@@ -177,26 +179,26 @@ class ModbusSimulatorServer:
         self.web_app.add_routes(
             [
                 web.get("/api/{tail:[a-z]*}", self.handle_html),
-                web.post("/api/{tail:[a-z]*}", self.handle_json),
+                web.post("/restapi/{tail:[a-z]*}", self.handle_json),
                 web.get("/{tail:[a-z0-9.]*}", self.handle_html_static),
                 web.get("/", self.handle_html_static),
             ]
         )
         self.web_app.on_startup.append(self.start_modbus_server)
         self.web_app.on_shutdown.append(self.stop_modbus_server)
-        self.generator_html = {
+        self.generator_html: dict[str, list] = {
             "log": ["", self.build_html_log],
             "registers": ["", self.build_html_registers],
             "calls": ["", self.build_html_calls],
             "server": ["", self.build_html_server],
         }
         self.generator_json = {
-            "log_json": [None, self.build_json_log],
-            "registers_json": [None, self.build_json_registers],
-            "calls_json": [None, self.build_json_calls],
-            "server_json": [None, self.build_json_server],
+            "log": self.build_json_log,
+            "registers": self.build_json_registers,
+            "calls": self.build_json_calls,
+            "server": self.build_json_server,
         }
-        self.submit = {
+        self.submit_html = {
             "Clear": self.action_clear,
             "Stop": self.action_stop,
             "Reset": self.action_reset,
@@ -210,20 +212,23 @@ class ModbusSimulatorServer:
             with open(html_file, encoding="utf-8") as handle:
                 self.generator_html[entry][0] = handle.read()
         self.refresh_rate = 0
-        self.register_filter: List[int] = []
-        self.call_list: List[tuple] = []
-        self.request_lookup = ServerDecoder.getFCdict()
+        self.register_filter: list[int] = []
+        self.call_list: list[CallTracer] = []
+        self.request_lookup = DecodePDU(True).lookup
         self.call_monitor = CallTypeMonitor()
         self.call_response = CallTypeResponse()
+        app_key = getattr(web, 'AppKey', str)  # fall back to str for aiohttp < 3.9.0
+        self.api_key = app_key("modbus_server")
 
     async def start_modbus_server(self, app):
         """Start Modbus server as asyncio task."""
         try:
             if getattr(self.modbus_server, "start", None):
                 await self.modbus_server.start()
-            app["modbus_server"] = asyncio.create_task(
+            app[self.api_key] = asyncio.create_task(
                 self.modbus_server.serve_forever()
             )
+            app[self.api_key].set_name("simulator modbus server")
         except Exception as exc:
             Log.error("Error starting modbus server, reason: {}", exc)
             raise exc
@@ -234,18 +239,19 @@ class ModbusSimulatorServer:
     async def stop_modbus_server(self, app):
         """Stop modbus server."""
         Log.info("Stopping modbus server")
-        app["modbus_server"].cancel()
+        await self.modbus_server.shutdown()
+        app[self.api_key].cancel()
         with contextlib.suppress(asyncio.exceptions.CancelledError):
-            await app["modbus_server"]
+            await app[self.api_key]
 
         Log.info("Modbus server Stopped")
 
     async def run_forever(self, only_start=False):
         """Start modbus and http servers."""
         try:
-            runner = web.AppRunner(self.web_app)
-            await runner.setup()
-            self.site = web.TCPSite(runner, self.http_host, self.http_port)
+            self.runner = web.AppRunner(self.web_app)
+            await self.runner.setup()
+            self.site = web.TCPSite(self.runner, self.http_host, self.http_port)
             await self.site.start()
         except Exception as exc:
             Log.error("Error starting http server, reason: {}", exc)
@@ -253,20 +259,23 @@ class ModbusSimulatorServer:
         Log.info("HTTP server started on ({}:{})", self.http_host, self.http_port)
         if only_start:
             return
-        while True:
-            await asyncio.sleep(1)
+        await self.serving
 
     async def stop(self):
         """Stop modbus and http servers."""
-        await self.site.stop()
+        await self.runner.cleanup()
         self.site = None
-        await asyncio.sleep(1)
+        if not self.serving.done():
+            self.serving.set_result(True)
+        await asyncio.sleep(0)
 
     async def handle_html_static(self, request):
         """Handle static html."""
         if not (page := request.path[1:]):
             page = "index.html"
-        file = os.path.join(self.web_path, page)
+        file = os.path.normpath(os.path.join(self.web_path, page))
+        if not file.startswith(self.web_path):
+            raise ValueError(f"File access outside {self.web_path} not permitted.")
         try:
             with open(file, encoding="utf-8"):
                 return web.FileResponse(file)
@@ -291,15 +300,18 @@ class ModbusSimulatorServer:
 
     async def handle_json(self, request):
         """Handle api registers."""
-        page_type = request.path.split("/")[-1]
-        params = await request.post()
-        json_dict = self.generator_html[page_type][0].copy()
-        result = self.generator_json[page_type][1](params, json_dict)
-        return web.Response(text=f"json build: {page_type} - {params} - {result}")
+        command = request.path.split("/")[-1]
+        params = await request.json()
+        try:
+            result = self.generator_json[command](params)
+        except (KeyError, ValueError, TypeError, IndexError) as exc:
+            Log.error("Unhandled error during json request: {}", exc)
+            return web.json_response({"result": "error", "error": f"Unhandled error Error: {exc}"})
+        return web.json_response(result)
 
     def build_html_registers(self, params, html):
         """Build html registers page."""
-        result_txt, foot = self.helper_build_html_submit(params)
+        result_txt, foot = self.helper_handle_submit(params, self.submit_html)
         if not result_txt:
             result_txt = "ok"
         if not foot:
@@ -342,9 +354,9 @@ class ModbusSimulatorServer:
         )
         return new_html
 
-    def build_html_calls(self, params, html):
+    def build_html_calls(self, params: dict, html: str) -> str:
         """Build html calls page."""
-        result_txt, foot = self.helper_build_html_submit(params)
+        result_txt, foot = self.helper_handle_submit(params, self.submit_html)
         if not foot:
             foot = "Montitoring active" if self.call_monitor.active else "not active"
         if not result_txt:
@@ -381,7 +393,7 @@ class ModbusSimulatorServer:
                 if function.function_code == self.call_monitor.function
                 else ""
             )
-            function_codes += f"<option value={function.function_code} {selected}>{function.function_code_name}</option>"
+            function_codes += f"<option value={function.function_code} {selected}>{function.function_code_name}</option>"  #type: ignore[attr-defined]
         simulation_action = (
             "ACTIVE" if self.call_response.active != RESPONSE_INACTIVE else ""
         )
@@ -392,7 +404,7 @@ class ModbusSimulatorServer:
         call_rows = ""
         for entry in reversed(self.call_list):
             # req_obj = self.request_lookup[entry[1]]
-            call_rows += f"<tr><td>{entry.call} - {entry.fc}</td><td>{entry.address}</td><td>{entry.count}</td><td>{entry.data}</td></tr>"
+            call_rows += f"<tr><td>{entry.call} - {entry.fc}</td><td>{entry.address}</td><td>{entry.count}</td><td>{entry.data.decode()}</td></tr>"
             # line += req_obj.funcion_code_name
         new_html = (
             html.replace("<!--SIMULATION_ACTIVE-->", simulation_action)
@@ -451,23 +463,154 @@ class ModbusSimulatorServer:
         """Build html server page."""
         return html
 
-    def build_json_registers(self, params, json_dict):
-        """Build html registers page."""
-        return f"json build registers: {params} - {json_dict}"
+    def build_json_registers(self, params):
+        """Build json registers response."""
+        # Process params using the helper function
+        result_txt, foot = self.helper_handle_submit(params, {
+            "Set": self.action_set,
+        })
 
-    def build_json_calls(self, params, json_dict):
-        """Build html calls page."""
-        return f"json build calls: {params} - {json_dict}"
+        if not result_txt:
+            result_txt = "ok"
+        if not foot:
+            foot = "Operation completed successfully"
 
-    def build_json_log(self, params, json_dict):
+        # Extract necessary parameters
+        try:
+            range_start = int(params.get("range_start", 0))
+            range_stop = int(params.get("range_stop", range_start))
+        except ValueError:
+            return {"result": "error", "error": "Invalid range parameters"}
+
+        # Retrieve register details
+        register_rows = []
+        for i in range(range_start, range_stop + 1):
+            inx, reg = self.datastore_context.get_text_register(i)
+            row = {
+                "index": inx,
+                "type": reg.type,
+                "access": reg.access,
+                "action": reg.action,
+                "value": reg.value,
+                "count_read": reg.count_read,
+                "count_write": reg.count_write
+            }
+            register_rows.append(row)
+
+        # Generate register types and actions (assume these are predefined mappings)
+        register_types = dict(self.datastore_context.registerType_name_to_id)
+        register_actions = dict(self.datastore_context.action_name_to_id)
+
+        # Build the JSON response
+        json_response = {
+            "result": result_txt,
+            "footer": foot,
+            "register_types": register_types,
+            "register_actions": register_actions,
+            "register_rows": register_rows,
+        }
+
+        return json_response
+
+    def build_json_calls(self, params: dict) -> dict:
+        """Build json calls response."""
+        result_txt, foot = self.helper_handle_submit(params, {
+            "Reset": self.action_reset,
+            "Add": self.action_add,
+            "Simulate": self.action_simulate,
+        })
+        if not foot:
+            foot = "Monitoring active" if self.call_monitor.active else "not active"
+        if not result_txt:
+            result_txt = "ok"
+
+        function_error = []
+        for i, txt in (
+            (1, "IllegalFunction"),
+            (2, "IllegalAddress"),
+            (3, "IllegalValue"),
+            (4, "SlaveFailure"),
+            (5, "Acknowledge"),
+            (6, "SlaveBusy"),
+            (7, "MemoryParityError"),
+            (10, "GatewayPathUnavailable"),
+            (11, "GatewayNoResponse"),
+        ):
+            function_error.append({
+                "value": i,
+                "text": txt,
+                "selected": i == self.call_response.error_response
+            })
+
+        range_start = (
+            self.call_monitor.range_start
+            if self.call_monitor.range_start != -1
+            else None
+        )
+        range_stop = (
+            self.call_monitor.range_stop
+            if self.call_monitor.range_stop != -1
+            else None
+        )
+
+        function_codes = []
+        for function in self.request_lookup.values():
+            function_codes.append({
+                "value": function.function_code,
+                "text": function.function_code_name,    # type: ignore[attr-defined]
+                "selected": function.function_code == self.call_monitor.function
+            })
+
+        simulation_action = "ACTIVE" if self.call_response.active != RESPONSE_INACTIVE else ""
+
+        max_len = MAX_FILTER if self.call_monitor.active else 0
+        while len(self.call_list) > max_len:
+            del self.call_list[0]
+        call_rows = []
+        for entry in reversed(self.call_list):
+            call_rows.append({
+                "call": entry.call,
+                "fc": entry.fc,
+                "address": entry.address,
+                "count": entry.count,
+                "data": entry.data.decode()
+            })
+
+        json_response = {
+            "simulation_action": simulation_action,
+            "range_start": range_start,
+            "range_stop": range_stop,
+            "function_codes": function_codes,
+            "function_show_hex_checked": self.call_monitor.hex,
+            "function_show_decoded_checked": self.call_monitor.decode,
+            "function_response_normal_checked": self.call_response.active == RESPONSE_NORMAL,
+            "function_response_error_checked": self.call_response.active == RESPONSE_ERROR,
+            "function_response_empty_checked": self.call_response.active == RESPONSE_EMPTY,
+            "function_response_junk_checked": self.call_response.active == RESPONSE_JUNK,
+            "function_response_split_checked": self.call_response.split > 0,
+            "function_response_split_delay": self.call_response.split,
+            "function_response_cr_checked": self.call_response.change_rate > 0,
+            "function_response_cr_pct": self.call_response.change_rate,
+            "function_response_delay": self.call_response.delay,
+            "function_response_junk": self.call_response.junk_len,
+            "function_error": function_error,
+            "function_response_clear_after": self.call_response.clear_after,
+            "call_rows": call_rows,
+            "foot": foot,
+            "result": result_txt
+        }
+
+        return json_response
+
+    def build_json_log(self, params):
         """Build json log page."""
-        return f"json build log: {params} - {json_dict}"
+        return {"result": "error", "error": "log endpoint not implemented", "params": params}
 
-    def build_json_server(self, params, json_dict):
+    def build_json_server(self, params):
         """Build html server page."""
-        return f"json build server: {params} - {json_dict}"
+        return {"result": "error", "error": "server endpoint not implemented", "params": params}
 
-    def helper_build_html_submit(self, params):
+    def helper_handle_submit(self, params, submit_actions):
         """Build html register submit."""
         try:
             range_start = int(params.get("range_start", -1))
@@ -477,9 +620,9 @@ class ModbusSimulatorServer:
             range_stop = int(params.get("range_stop", range_start))
         except ValueError:
             range_stop = -1
-        if (submit := params["submit"]) not in self.submit:
+        if (submit := params["submit"]) not in submit_actions:
             return None, None
-        return self.submit[submit](params, range_start, range_stop)
+        return submit_actions[submit](params, range_start, range_stop)
 
     def action_clear(self, _params, _range_start, _range_stop):
         """Clear register filter."""
@@ -533,7 +676,7 @@ class ModbusSimulatorServer:
         self.call_monitor.range_start = range_start
         self.call_monitor.range_stop = range_stop
         self.call_monitor.function = (
-            int(params["function"]) if params["function"] else ""
+            int(params["function"]) if params["function"] else -1
         )
         self.call_monitor.hex = "show_hex" in params
         self.call_monitor.decode = "show_decode" in params
@@ -601,7 +744,7 @@ class ModbusSimulatorServer:
                 fc=response.function_code,
                 address=response.address if hasattr(response, "address") else -1,
                 count=response.count if hasattr(response, "count") else -1,
-                data="-",
+                data=b"-",
             )
             self.call_list.append(tracer)
             self.call_monitor.trace_response = False
@@ -612,14 +755,14 @@ class ModbusSimulatorServer:
         skip_encoding = False
         if self.call_response.active == RESPONSE_EMPTY:
             Log.warning("Sending empty response")
-            response.should_respond = False
-        elif self.call_response.active == RESPONSE_NORMAL:
+            return None, False
+        if self.call_response.active == RESPONSE_NORMAL:
             if self.call_response.delay:
                 Log.warning(
                     "Delaying response by {}s for all incoming requests",
                     self.call_response.delay,
                 )
-                time.sleep(self.call_response.delay)  # change to async
+                sleep(self.call_response.delay)  # change to async
             else:
                 pass
                 # self.call_response.change_rate
@@ -648,7 +791,7 @@ class ModbusSimulatorServer:
         """
         if self.call_monitor.function not in {-1, request.function_code}:
             return
-        address = (request.address if hasattr(request, "address") else -1,)
+        address = request.address if hasattr(request, "address") else -1
         if self.call_monitor.range_start != -1 and address != -1:
             if (
                 self.call_monitor.range_start > address
@@ -660,7 +803,7 @@ class ModbusSimulatorServer:
             fc=request.function_code,
             address=address,
             count=request.count if hasattr(request, "count") else -1,
-            data="-",
+            data=b"-",
         )
         self.call_list.append(tracer)
         self.call_monitor.trace_response = True
